@@ -4,10 +4,11 @@ use cursive::event::{Event, EventResult, Key};
 use cursive::theme::{ColorStyle, PaletteColor};
 use regex::{Match, Regex};
 
+use crate::{ContainerManager, ReadingInfo};
 use crate::book::Book;
 use crate::common::{byte_index_for_char, char_index_for_byte};
+use crate::container::Container;
 use crate::controller::{ReverseInfo, update_status_callback};
-use crate::ReadingInfo;
 use crate::view::han::Han;
 use crate::view::xi::Xi;
 
@@ -29,6 +30,8 @@ struct TraceInfo {
 
 pub struct ReadingView {
 	render: Box<dyn Render>,
+	container_manager: ContainerManager,
+	container: Box<dyn Container>,
 	book: Box<dyn Book>,
 	reading: ReadingInfo,
 	search_pattern: Option<String>,
@@ -133,63 +136,12 @@ impl View for ReadingView {
 	}
 
 	fn on_event(&mut self, e: Event) -> EventResult {
-		let result = match e {
-			Event::Char(' ') | Event::Key(Key::PageDown) => self.next_page(),
-			Event::Char('b') | Event::Key(Key::PageUp) => self.prev_page(),
-			Event::Key(Key::Home) => {
-				self.reading.line = 0;
-				self.reading.position = 0;
-				self.render.redraw(self.book.lines(), &mut self.reading, &mut self.render_context);
-				self.push_trace(true);
-				Ok(())
-			}
-			Event::Key(Key::End) => {
-				self.reading.line = self.book.lines().len();
-				self.reading.position = 0;
-				self.render.prev(self.book.lines(), &mut self.reading, &mut self.render_context);
-				self.push_trace(true);
-				Ok(())
-			}
-			Event::Key(Key::Down) => {
-				if self.render_context.next.is_some() {
-					self.render.next_line(self.book.lines(), &mut self.reading, &mut self.render_context);
-				}
-				self.push_trace(true);
-				Ok(())
-			}
-			Event::Key(Key::Up) => {
-				self.render.prev_line(self.book.lines(), &mut self.reading, &mut self.render_context);
-				self.push_trace(true);
-				Ok(())
-			}
-			Event::Char('n') => {
-				let (line, position) = match &self.reading.reverse {
-					Some(reverse) => (reverse.line, reverse.end),
-					None => (self.reading.line, self.reading.position),
-				};
-				self.search_next(line, position)
-			}
-			Event::Char('N') => {
-				let (line, position) = match &self.reading.reverse {
-					Some(reverse) => (reverse.line, reverse.start),
-					None => (self.reading.line, self.reading.position),
-				};
-				self.search_prev(line, position)
-			}
-			Event::CtrlChar('d') => self.switch_chapter_internal(self.reading.chapter + 1),
-			Event::CtrlChar('b') => {
-				if self.reading.chapter > 0 {
-					self.switch_chapter_internal(self.reading.chapter - 1)
-				} else {
-					Ok(())
-				}
-			}
-			Event::Key(Key::Right) => self.goto_trace(false),
-			Event::Key(Key::Left) => self.goto_trace(true),
-			_ => { return EventResult::Ignored; }
-		};
-		let status = match result {
-			Ok(..) => self.status_msg(),
+		let status = match self.process_event(e) {
+			Ok(consumed) => if consumed {
+				self.status_msg()
+			} else {
+				return EventResult::Ignored;
+			},
 			Err(e) => e.to_string(),
 		};
 		EventResult::Consumed(Some(update_status_callback(status)))
@@ -197,11 +149,16 @@ impl View for ReadingView {
 }
 
 impl ReadingView {
-	pub(crate) fn new(book: Box<dyn Book>, render_type: &String, reading: ReadingInfo, search_pattern: &Option<String>) -> Result<ReadingView> {
+	pub(crate) fn new(render_type: &String, mut reading: ReadingInfo, search_pattern: &Option<String>) -> Result<ReadingView> {
+		let container_manager = Default::default();
+		let mut container = load_container(&container_manager, &reading)?;
+		let book = load_book(&container_manager, &mut container, &mut reading)?;
 		let render_context = RenderContext::build(&book);
 		let render: Box<dyn Render> = load_render(render_type);
 		let trace = vec![TraceInfo { chapter: reading.chapter, line: reading.line, position: reading.position }];
 		Ok(ReadingView {
+			container_manager,
+			container,
 			book,
 			reading,
 			render,
@@ -215,7 +172,15 @@ impl ReadingView {
 	pub(crate) fn status_msg(&self) -> String {
 		let title = match self.book.title() {
 			Some(t) => t,
-			None => &self.reading.filename,
+			None => {
+				let names = self.container.inner_book_names();
+				if names.len() == 1 {
+					&self.reading.filename
+				} else {
+					let name = &names[self.reading.inner_book];
+					name.name()
+				}
+			}
 		};
 		format!("{}({}:{})", title, self.book.lines().len(), self.reading.line)
 	}
@@ -237,14 +202,35 @@ impl ReadingView {
 		&self.book
 	}
 
-	pub(crate) fn switch_book(&mut self, book: Box<dyn Book>, reading: ReadingInfo) -> String {
+	pub(crate) fn reading_container(&self) -> &Box<dyn Container> {
+		&self.container
+	}
+
+	pub(crate) fn switch_container(&mut self, mut reading: ReadingInfo) -> Result<String> {
+		let mut container = load_container(&self.container_manager, &reading)?;
+		let book = load_book(&self.container_manager, &mut container, &mut reading)?;
+		self.container = container;
+		self.book = book;
+		self.reading = reading;
+		self.render.redraw(&self.book.lines(), &self.reading, &mut self.render_context);
+		Ok(self.status_msg())
+	}
+
+	pub(crate) fn switch_book(&mut self, reading: ReadingInfo) -> String {
+		match self.do_switch_book(reading) {
+			Ok(..) => self.status_msg(),
+			Err(e) => e.to_string(),
+		}
+	}
+	fn do_switch_book(&mut self, mut reading: ReadingInfo) -> Result<()> {
+		let book = load_book(&self.container_manager, &mut self.container, &mut reading)?;
 		self.book = book;
 		self.reading = reading;
 		self.trace.clear();
 		self.trace.push(TraceInfo { chapter: self.reading.chapter, line: self.reading.line, position: self.reading.position });
 		self.current_trace = 0;
 		self.render.redraw(self.book.lines(), &mut self.reading, &mut self.render_context);
-		self.status_msg()
+		Ok(())
 	}
 
 	pub(crate) fn goto_line(&mut self, line: usize) -> Result<()> {
@@ -264,20 +250,97 @@ impl ReadingView {
 		self.render = load_render(render_type);
 	}
 
-	fn next_page(&mut self) -> Result<()> {
+	fn process_event(&mut self, e: Event) -> Result<bool> {
+		match e {
+			Event::Char(' ') | Event::Key(Key::PageDown) => { self.next_page()?; }
+			Event::Key(Key::PageUp) => { self.prev_page()?; }
+			Event::Key(Key::Home) => {
+				self.reading.line = 0;
+				self.reading.position = 0;
+				self.render.redraw(self.book.lines(), &mut self.reading, &mut self.render_context);
+				self.push_trace(true);
+			}
+			Event::Key(Key::End) => {
+				self.reading.line = self.book.lines().len();
+				self.reading.position = 0;
+				self.render.prev(self.book.lines(), &mut self.reading, &mut self.render_context);
+				self.push_trace(true);
+			}
+			Event::Key(Key::Down) => {
+				if self.render_context.next.is_some() {
+					self.render.next_line(self.book.lines(), &mut self.reading, &mut self.render_context);
+				}
+				self.push_trace(true);
+			}
+			Event::Key(Key::Up) => {
+				self.render.prev_line(self.book.lines(), &mut self.reading, &mut self.render_context);
+				self.push_trace(true);
+			}
+			Event::Char('n') => {
+				let (line, position) = match &self.reading.reverse {
+					Some(reverse) => (reverse.line, reverse.end),
+					None => (self.reading.line, self.reading.position),
+				};
+				self.search_next(line, position)?;
+			}
+			Event::Char('N') => {
+				let (line, position) = match &self.reading.reverse {
+					Some(reverse) => (reverse.line, reverse.start),
+					None => (self.reading.line, self.reading.position),
+				};
+				self.search_prev(line, position)?;
+			}
+			Event::CtrlChar('d') => {
+				self.switch_chapter_internal(self.reading.chapter + 1)?;
+			}
+			Event::CtrlChar('b') => {
+				if self.reading.chapter > 0 {
+					self.switch_chapter_internal(self.reading.chapter - 1)?;
+				}
+			}
+			Event::Key(Key::Right) => self.goto_trace(false)?,
+			Event::Key(Key::Left) => self.goto_trace(true)?,
+			_ => return Ok(false),
+		};
+		Ok(true)
+	}
+
+	fn next_page(&mut self) -> Result<bool> {
 		match &self.render_context.next {
 			Some(next) => {
 				self.reading.line = next.line;
 				self.reading.position = next.position;
 				self.render.redraw(self.book.lines(), &mut self.reading, &mut self.render_context);
+				self.push_trace(true);
+				Ok(true)
+			}
+			None => {
+				if self.switch_chapter_internal(self.reading.chapter + 1)? {
+					Ok(true)
+				} else {
+					let book_index = self.reading.inner_book + 1;
+					let book_count = self.container.inner_book_names().len();
+					if book_index >= book_count {
+						Ok(false)
+					} else {
+						let reading = ReadingInfo {
+							filename: self.reading.filename.clone(),
+							inner_book: book_index,
+							chapter: 0,
+							line: 0,
+							position: 0,
+							ts: 0,
+							reverse: None,
+						};
+						self.do_switch_book(reading)?;
+						Ok(true)
+					}
+				}
 			}
-			None => return self.switch_chapter_internal(self.reading.chapter + 1),
 		}
-		self.push_trace(true);
-		Ok(())
 	}
 
-	fn prev_page(&mut self) -> Result<()> {
+	fn prev_page(&mut self) -> Result<bool> {
 		if self.reading.line == 0 && self.reading.position == 0 {
 			let reading = &mut self.reading;
 			if reading.chapter > 0 {
@@ -289,12 +352,38 @@ impl ReadingView {
 				reading.position = 0;
 				// prev need decrease this invalid reading.line
 				self.render.prev(book.lines(), reading, &mut self.render_context);
+				self.push_trace(true);
+				Ok(true)
+			} else {
+				if reading.inner_book > 0 {
+					let mut new_reading = ReadingInfo {
+						filename: reading.filename.clone(),
+						inner_book: reading.inner_book - 1,
+						chapter: usize::MAX,
+						line: 0,
+						position: 0,
+						ts: 0,
+						reverse: None,
+					};
+					self.book = load_book(&self.container_manager, &mut self.container, &mut new_reading)?;
+					new_reading.chapter = self.book.current_chapter();
+					new_reading.line = self.book.lines().len();
+					new_reading.position = self.book.lines()[new_reading.line - 1].chars().count();
+					self.reading = new_reading;
+					self.prev_page()?;
+					self.trace.clear();
+					self.trace.push(TraceInfo { chapter: self.reading.chapter, line: self.reading.line, position: self.reading.position });
+					self.current_trace = 0;
+					Ok(true)
+				} else {
+					Ok(false)
+				}
 			}
 		} else {
 			self.render.prev(self.book.lines(), &mut self.reading, &mut self.render_context);
+			self.push_trace(true);
+			Ok(true)
 		}
-		self.push_trace(true);
-		Ok(())
 	}
 
 	fn build_reverse(text: &str, line: usize, m: Match) -> Option<ReverseInfo> {
@@ -311,7 +400,7 @@ impl ReadingView {
 			Err(e) => e.to_string(),
 		}
 	}
-	fn switch_chapter_internal(&mut self, chapter: usize) -> Result<()> {
+	fn switch_chapter_internal(&mut self, chapter: usize) -> Result<bool> {
 		if chapter < self.book.chapter_count() {
 			self.book.set_chapter(chapter)?;
 			self.reading.chapter = chapter;
@@ -319,8 +408,10 @@ impl ReadingView {
 			self.reading.position = 0;
 			self.render.redraw(self.book.lines(), &mut self.reading, &mut self.render_context);
 			self.push_trace(true);
+			Ok(true)
+		} else {
+			Ok(false)
 		}
-		Ok(())
 	}
 
 	fn search_next(&mut self, start_line: usize, start_position: usize) -> Result<()> {
@@ -430,3 +521,25 @@ impl ReadingView {
 		Ok(())
 	}
 }
+
+fn load_container(container_manager: &ContainerManager, reading: &ReadingInfo) -> Result<Box<dyn Container>> {
+	let container = container_manager.open(&reading.filename)?;
+	let book_names = container.inner_book_names();
+	if book_names.len() == 0 {
+		return Err(anyhow!("No content supported."));
+	}
+	Ok(container)
+}
+
+fn load_book(container_manager: &ContainerManager, container: &mut Box<dyn Container>, reading: &mut ReadingInfo) -> Result<Box<dyn Book>> {
+	let book = container_manager.load_book(container, reading.inner_book, reading.chapter)?;
+	let lines = book.lines();
+	if reading.line >= lines.len() {
+		reading.line = lines.len() - 1;
+	}
+	let chars = lines[reading.line].chars().count();
+	if reading.position >= chars {
+		reading.position = 0;
+	}
+	Ok(book)
+}
\ No newline at end of file
